@@ -93,14 +93,13 @@ int get_unsigned(unsigned *val, const char *arg, int base)
 }
 
 /*
- * get_jiffies is "translated" from a similar routine "get_time" in
- * tc_util.c.  we don't use the exact same routine because tc passes
- * microseconds to the kernel and the callers of get_jiffies want 
- * to pass jiffies, and have a different assumption for the units of
- * a "raw" number.
+ * get_time_rtt is "translated" from a similar routine "get_time" in
+ * tc_util.c.  We don't use the exact same routine because tc passes
+ * microseconds to the kernel and the callers of get_time_rtt want to
+ * pass milliseconds (standard unit for rtt values since 2.6.27), and
+ * have a different assumption for the units of a "raw" number.
  */
-
-int get_jiffies(unsigned *jiffies, const char *arg, int base, int *raw)
+int get_time_rtt(unsigned *val, const char *arg, int *raw)
 {
 	double t;
 	unsigned long res;
@@ -112,35 +111,22 @@ int get_jiffies(unsigned *jiffies, const char *arg, int base, int *raw)
 			return -1;
 	}
 	else {
-		res = strtoul(arg,&p,base);
+		res = strtoul(arg, &p, 0);
 		if (res > UINT_MAX)
 			return -1;
 		t = (double)res;
 	}
 	if (p == arg)
 		return -1;
-
-	if (__iproute2_hz_internal == 0)
-                __iproute2_hz_internal = __get_hz();
-	
 	*raw = 1;
 
 	if (*p) {
 		*raw = 0;
                 if (strcasecmp(p, "s") == 0 || strcasecmp(p, "sec")==0 ||
                     strcasecmp(p, "secs")==0)
-                        t *= __iproute2_hz_internal;
+                        t *= 1000;
                 else if (strcasecmp(p, "ms") == 0 || strcasecmp(p, "msec")==0 ||
                          strcasecmp(p, "msecs") == 0)
-                        t *= __iproute2_hz_internal/1000.0;
-                else if (strcasecmp(p, "us") == 0 || strcasecmp(p, "usec")==0 ||
-                         strcasecmp(p, "usecs") == 0)
-                        t *= __iproute2_hz_internal/1000000.0;
-                else if (strcasecmp(p, "ns") == 0 || strcasecmp(p, "nsec")==0 ||
-                         strcasecmp(p, "nsecs") == 0)
-                        t *= __iproute2_hz_internal/1000000000.0;
-		else if (strcasecmp(p, "j") == 0 || strcasecmp(p, "hz") == 0 ||
-			 strcasecmp(p,"jiffies") == 0)
 			t *= 1.0; /* allow suffix, do nothing */
                 else
                         return -1;
@@ -148,9 +134,9 @@ int get_jiffies(unsigned *jiffies, const char *arg, int base, int *raw)
 
 	/* emulate ceil() without having to bring-in -lm and always be >= 1 */
 
-	*jiffies = t;
-	if (*jiffies < t)
-		*jiffies += 1;
+	*val = t;
+	if (*val < t)
+		*val += 1;
 	
         return 0;
 
@@ -714,7 +700,7 @@ ssize_t getcmdline(char **linep, size_t *lenp, FILE *in)
 	while ((cp = strstr(*linep, "\\\n")) != NULL) {
 		char *line1 = NULL;
 		size_t len1 = 0;
-		size_t cc1;
+		ssize_t cc1;
 
 		if ((cc1 = getline(&line1, &len1, in)) < 0) {
 			fprintf(stderr, "Missing continuation line\n");
